#!/usr/bin/env python
# -*- coding: utf-8 -*-

import ldap
from django_auth_ldap.config import LDAPSearch, PosixGroupType

"""
Django settings for henry2web project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

AUTH_USER_MODEL = 'ponto.Funcionario'

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'ddk$b7b6p(bj^$cumb60l6i+bnew5-zh@uxn9ng!%)*nebfe_o'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = ['ponto.dimap.ufrn.br']

# Application definition


INSTALLED_APPS = (
	'django_admin_bootstrapped',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sessions',
#    'south',
    'ponto',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
)

ROOT_URLCONF = 'henry2web.urls'

WSGI_APPLICATION = 'henry2web.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'deposito.db'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'pt_BR'

TIME_ZONE = 'America/Fortaleza'

USE_I18N = True

USE_L10N = True

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(BASE_DIR, "henry2web", 'static')

MEDIA_ROOT = os.path.join(BASE_DIR, "henry2web", 'media')

MEDIA_URL = '/media/'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)


TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

TEMPLATE_DIRS = (
	os.path.join(BASE_DIR, 'henry2web','templates'),
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

#SESSION_ENGINE = "django.contrib.sessions.backends.cache"

#### Autenticação LDAP

AUTHENTICATION_BACKENDS = (
	'django.contrib.auth.backends.ModelBackend',
    'django_auth_ldap.backend.LDAPBackend',
)

# Dados do servidor

AUTH_LDAP_SERVER_URI = "ldap://apps.dimap.ufrn.br"

AUTH_LDAP_BIND_DN = "cn=cliente,dc=dimap,dc=ufrn,dc=br"
AUTH_LDAP_BIND_PASSWORD = "#tf01btjuf"
AUTH_LDAP_USER_SEARCH = LDAPSearch("ou=People,dc=dimap,dc=ufrn,dc=br", ldap.SCOPE_SUBTREE, "(uid=%(user)s)")

# Mapeamentos de atributos

AUTH_LDAP_USER_ATTR_MAP = {
	"username": "uid",
	"password": "userPassword",
#    "first_name": "cn",
#    "last_name": "sn",
}

# Informações dos grupos

#AUTH_LDAP_MIRROR_GROUPS = True
#AUTH_LDAP_ALWAYS_UPDATE_USER = True
AUTH_LDAP_GROUP_TYPE = PosixGroupType()
AUTH_LDAP_GROUP_SEARCH = LDAPSearch("ou=Groups,dc=dimap,dc=ufrn,dc=br", ldap.SCOPE_SUBTREE, "(objectClass=posixGroup)")

AUTH_LDAP_USER_FLAGS_BY_GROUP = {
    "is_active": "cn=users,ou=Group,dc=dimap,dc=ufrn,dc=br",
    "is_staff": "cn=users,ou=Group,dc=dimap,dc=ufrn,dc=br",
    #"is_superuser": "ou=People,dc=dimap,dc=ufrn,dc=br"
}

import logging

logger = logging.getLogger('django_auth_ldap')
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


#SESSION_COOKIE_SECURE = True
#CSRF_COOKIE_SECURE = True

#os.environ['HTTPS'] = "on"


